package sets;


// node in a singly-linked chain
interface Node<T>
{
    // value of node
    T item();

    // hash code of node's value
    int key();

    // reference to next node in chain
    Node<T> next();

    // is node valid?
    boolean valid();

    // lock, unlock node
    void lock();
    void unlock();

    // set value of node to `item'
    void setItem(T item);

    // set reference to next node to `next'
    void setNext(Node<T> next);

    // set node to valid, invalid; return true iff successful
    boolean setValid();
    boolean setInvalid();

    // do atomically:
    //   if next node == expectNext and node is valid,
    //   then set next node to newNext, the nore remains valid, and return true;
    //   else return false
    boolean setNextIfValid(Node<T> expectNext, Node<T> newNext);

    // next and valid fields of node, copied atomically and immutable
    Node<T> nextValid();
}
